@@ -25,8 +25,11 @@ pygtk.require('2.0')
 import gobject
 import gtk.glade
 import threading
+import xml.sax.saxutils as saxutils
 import os
-from gnome import url_show
+import locale
+import re
+import datetime
 
 #our own imports
 import GTG
@@ -34,8 +37,7 @@ from GTG.taskeditor.editor            import TaskEditor
 from GTG.taskbrowser.CellRendererTags import CellRendererTags
 from GTG.taskbrowser                  import GnomeConfig
 from GTG.taskbrowser                  import treetools
-from GTG.tools                        import colors
-from GTG.core                         import CoreConfig
+from GTG.tools                        import colors, openurl
 
 #=== OBJECTS ===================================================================
 
@@ -45,6 +47,7 @@ from GTG.core                         import CoreConfig
 WORKVIEW      = False
 SIDEBAR       = False
 CLOSED_PANE   = False
+TOOLBAR       = True
 QUICKADD_PANE = True
 
 EXPERIMENTAL_NOTES = False
@@ -92,15 +95,31 @@ class TaskBrowser:
         self.tagpopup           = self.wTree.get_widget("TagContextMenu")
         self.taskpopup          = self.wTree.get_widget("TaskContextMenu")
         self.closedtaskpopup    = self.wTree.get_widget("ClosedTaskContextMenu")
+        self.editbutton         = self.wTree.get_widget("edit_b")
         self.donebutton         = self.wTree.get_widget("mark_as_done_b")
+        self.newtask            = self.wTree.get_widget("new_task_b")
+        self.newsubtask         = self.wTree.get_widget("new_subtask_b")
         self.dismissbutton      = self.wTree.get_widget("dismiss")
+        self.deletebutton      = self.wTree.get_widget("delete_b")
         self.about              = self.wTree.get_widget("aboutdialog1")
-
+        self.edit_mi            = self.wTree.get_widget("edit_mi")
+        
+        #Set the tooltip for the toolbar buttons
+        self.donebutton.set_tooltip_text(GnomeConfig.MARK_DONE_TOOLTIP)
+        self.editbutton.set_tooltip_text(_("Edit the selected task"))
+        self.dismissbutton.set_tooltip_text(GnomeConfig.MARK_DISMISS_TOOLTIP)
+        self.newtask.set_tooltip_text(_("Create a new task"))
+        self.newsubtask.set_tooltip_text(_("Create a new subtask"))
+        self.deletebutton.set_tooltip_text(GnomeConfig.DELETE_TOOLTIP)
+  
         # Initialize "About" dialog
-        gtk.about_dialog_set_url_hook(lambda dialog, url: url_show(url))
+        gtk.about_dialog_set_url_hook(lambda dialog, url: openurl.openurl(url))
         self.about.set_website(GTG.URL)
         self.about.set_website_label(GTG.URL)
         self.about.set_version(GTG.VERSION)
+        self.about.set_authors(GTG.AUTHORS)
+        self.about.set_artists(GTG.ARTISTS)
+        self.about.set_translator_credits(GTG.TRANSLATORS)
         
         # Initialize menu
 
@@ -123,7 +142,9 @@ class TaskBrowser:
                 "on_delete_cancel"    : lambda x : x.hide,
                 "on_add_subtask"      : self.on_add_subtask,
                 "on_closed_task_treeview_button_press_event" : self.on_closed_task_treeview_button_press_event,
+                "on_closed_task_treeview_key_press_event" : self.on_closed_task_treeview_key_press_event,
                 "on_task_treeview_button_press_event" : self.on_task_treeview_button_press_event,
+                "on_task_treeview_key_press_event" : self.on_task_treeview_key_press_event,
                 "on_tag_treeview_button_press_event"  : self.on_tag_treeview_button_press_event,
                 "on_colorchooser_activate"            : self.on_colorchooser_activate,
                 "on_workview_toggled"                 : self.on_workview_toggled,
@@ -134,6 +155,7 @@ class TaskBrowser:
                 "on_bg_color_toggled"                 : self.on_bg_color_toggled,
                 "on_quickadd_field_activate"          : self.quickadd,
                 "on_quickadd_button_activate"         : self.quickadd,
+                "on_view_toolbar_toggled"             : self.on_toolbar_toggled,
                 "on_view_quickadd_toggled"            : self.toggle_quickadd,
                 "on_about_clicked"                    : self.on_about_clicked,
                 "on_about_close"                      : self.on_about_close,
@@ -151,7 +173,10 @@ class TaskBrowser:
         if not self.notes :
             self.note_toggle.hide()
             self.new_note_button.hide()
-        
+            
+        #Set the tooltip for the toolbar button
+        self.new_note_button.set_tooltip_text("Create a new note")
+                
         # Model constants
         self.TASK_MODEL_OBJ         = 0
         self.TASK_MODEL_TITLE       = 1
@@ -177,7 +202,7 @@ class TaskBrowser:
         #The tview and their model
         self.taskdone_tview = self.wTree.get_widget("taskdone_tview")
         self.taskdone_ts    = gtk.TreeStore(gobject.TYPE_PYOBJECT, str,str,str,str,str,gobject.TYPE_PYOBJECT)
-        #self.note_tview = self.wTree.get_widget("note_tview")
+        self.note_tview = self.wTree.get_widget("note_tview")
         self.note_tview = gtk.TreeView()
         self.note_tview.connect("row-activated",self.on_edit_note)
         self.note_tview.show()
@@ -201,9 +226,13 @@ class TaskBrowser:
         self.toggle_workview = self.wTree.get_widget("workview_toggle")
         self.quickadd_entry = self.wTree.get_widget("quickadd_field")
         
+        #Set the tooltip for the toolbar button
+        self.toggle_workview.set_tooltip_text("Toggle the work view")
+        
         #The panes
         self.sidebar       = self.wTree.get_widget("sidebar")
         self.closed_pane   = self.wTree.get_widget("closed_pane")
+        self.toolbar       = self.wTree.get_widget("task_tb")
         self.quickadd_pane = self.wTree.get_widget("quickadd_pane")
                
         #The tid that will be deleted
@@ -225,13 +254,46 @@ class TaskBrowser:
         self.menu_view_workview.set_active(WORKVIEW)
         self.wTree.get_widget("view_sidebar").set_active(SIDEBAR)
         self.wTree.get_widget("view_closed").set_active(CLOSED_PANE)
+        self.wTree.get_widget("view_toolbar").set_active(TOOLBAR)
         self.wTree.get_widget("view_quickadd").set_active(QUICKADD_PANE)
         self.priv["bg_color_enable"] = True
     
-		#connecting the refresh signal from the requester
+        #connecting the refresh signal from the requester
         self.lock = threading.Lock()
         self.req.connect("refresh",self.do_refresh)
+        
+        agr = gtk.AccelGroup()
+        self.wTree.get_widget("MainWindow").add_accel_group(agr)
 
+        # define accelerator keys
+        view_sidebar = self.wTree.get_widget("view_sidebar")
+        key, mod = gtk.accelerator_parse("F9")
+        view_sidebar.add_accelerator("activate", agr, key, mod, gtk.ACCEL_VISIBLE)
+
+        file_quit = self.wTree.get_widget("file_quit")
+        key, mod = gtk.accelerator_parse("<Control>q")
+        file_quit.add_accelerator("activate", agr, key, mod, gtk.ACCEL_VISIBLE)
+
+        edit_undo = self.wTree.get_widget("edit_undo")
+        key, mod = gtk.accelerator_parse("<Control>z")
+        edit_undo.add_accelerator("activate", agr, key, mod, gtk.ACCEL_VISIBLE)
+
+        edit_redo = self.wTree.get_widget("edit_redo")
+        key, mod = gtk.accelerator_parse("<Control>y")
+        edit_redo.add_accelerator("activate", agr, key, mod, gtk.ACCEL_VISIBLE)
+
+        new_task_mi = self.wTree.get_widget("new_task_mi")
+        key, mod = gtk.accelerator_parse("<Control>n")
+        new_task_mi.add_accelerator("activate", agr, key, mod, gtk.ACCEL_VISIBLE)
+        
+        new_subtask_mi = self.wTree.get_widget("new_subtask_mi")
+        key, mod = gtk.accelerator_parse("<Control><Shift>n")
+        new_subtask_mi.add_accelerator("activate", agr, key, mod, gtk.ACCEL_VISIBLE)
+        
+        edit_button = self.wTree.get_widget("edit_b")
+        key, mod = gtk.accelerator_parse("<Control>e")
+        edit_button.add_accelerator("clicked", agr, key, mod, gtk.ACCEL_VISIBLE)
+        
     def __restore_state_from_conf(self):
         
         # Extract state from configuration dictionary
@@ -254,11 +316,12 @@ class TaskBrowser:
         if self.config["browser"].has_key("tag_pane"):
             tag_pane         = eval(self.config["browser"]["tag_pane"])
             if not tag_pane:
-                self.sidebar.hide()
                 self.wTree.get_widget("view_sidebar").set_active(False)
+                self.sidebar.hide()
             else:
-                self.sidebar.show()
                 self.wTree.get_widget("view_sidebar").set_active(True)
+                self.sidebar.show()
+
                 
         if self.config["browser"].has_key("closed_task_pane"):
             closed_task_pane = eval(self.config["browser"]["closed_task_pane"])
@@ -272,6 +335,12 @@ class TaskBrowser:
         if self.config["browser"].has_key("ctask_pane_height"):
             ctask_pane_height = eval(self.config["browser"]["ctask_pane_height"])
             self.wTree.get_widget("vpaned1").set_position(ctask_pane_height)
+            
+        if self.config["browser"].has_key("toolbar"):
+            toolbar    = eval(self.config["browser"]["toolbar"])
+            if not toolbar    :
+                self.toolbar.hide()
+                self.wTree.get_widget("view_toolbar").set_active(False)
                 
         if self.config["browser"].has_key("quick_add"):
             quickadd_pane    = eval(self.config["browser"]["quick_add"])
@@ -279,6 +348,7 @@ class TaskBrowser:
                 self.quickadd_pane.hide()
                 self.wTree.get_widget("view_quickadd").set_active(False)
                 
+                
         if self.config["browser"].has_key("bg_color_enable"):
             bgcol_enable = eval(self.config["browser"]["bg_color_enable"])
             self.priv["bg_color_enable"] = bgcol_enable
@@ -335,8 +405,9 @@ class TaskBrowser:
         # Get configuration values
         tag_sidebar     = self.sidebar.get_property("visible")
         closed_pane     = self.closed_pane.get_property("visible")
+        toolbar         = self.toolbar.get_property("visible")
         quickadd_pane   = self.quickadd_pane.get_property("visible")
-        task_tv_sort_id = self.task_ts.get_sort_column_id()
+        #task_tv_sort_id = self.task_ts.get_sort_column_id()
         sort_column     = self.priv["tasklist"]["sort_column"]
         sort_order      = self.priv["tasklist"]["sort_order"]
         closed_pane_height = self.wTree.get_widget("vpaned1").get_position()
@@ -353,6 +424,7 @@ class TaskBrowser:
         self.config["browser"]["tag_pane"]          = tag_sidebar
         self.config["browser"]["closed_task_pane"]  = closed_pane
         self.config["browser"]["ctask_pane_height"] = closed_pane_height
+        self.config["browser"]["toolbar"]           = toolbar
         self.config["browser"]["quick_add"]         = quickadd_pane
         self.config["browser"]["bg_color_enable"]   = self.priv["bg_color_enable"]
         self.config["browser"]["collapsed_tasks"]   = self.priv["collapsed_tid"]
@@ -365,10 +437,6 @@ class TaskBrowser:
         self.config["browser"]["view"]              = view
         if self.notes :
             self.config["browser"]["experimental_notes"] = True
-             
-    def on_close(self):
-        self.__save_state_to_conf()
-        self.close()
  
     def main(self):
         #Here we will define the main TaskList interface
@@ -425,6 +493,15 @@ class TaskBrowser:
               }
         wTree.signal_autoconnect(dic)
         window = wTree.get_widget("ColorChooser")
+	# Get previous color
+        tags,notag_only = self.get_selected_tags() #pylint: disable-msg=W0612
+        if len(tags) == 1:
+                color = tags[0].get_attribute("color")
+                if color != None:
+			colorspec = gtk.gdk.Color(color)
+			colorsel = window.colorsel
+			colorsel.set_previous_color(colorspec)
+			colorsel.set_current_color(colorspec)
         window.show()
 
     def on_color_response(self,widget,response) :
@@ -444,8 +521,8 @@ class TaskBrowser:
     
     def do_toggle_workview(self):
         #We have to be careful here to avoid a loop of signals
-        menu_state   = self.menu_view_workview.get_active()
-        button_state = self.toggle_workview.get_active()
+        #menu_state   = self.menu_view_workview.get_active()
+        #button_state = self.toggle_workview.get_active()
         #We cannot have note and workview at the same time
         if not self.workview and self.note_toggle.get_active() :
             self.note_toggle.set_active(False)
@@ -457,10 +534,13 @@ class TaskBrowser:
         self.do_refresh()
 
     def on_sidebar_toggled(self,widget) :
-        if widget.get_active() :
-            self.sidebar.show()
-        else :
+        view_sidebar = self.wTree.get_widget("view_sidebar")
+        if self.sidebar.get_property("visible"):
+            view_sidebar.set_active(False)
             self.sidebar.hide()
+        else :
+            view_sidebar.set_active(True)
+            self.sidebar.show()
             
     def on_note_toggled(self,widget) :
         self.noteview = not self.noteview
@@ -482,52 +562,202 @@ class TaskBrowser:
             self.priv["bg_color_enable"] = False
         self.do_refresh()
             
+    def on_toolbar_toggled(self,widget) :
+        if widget.get_active() :
+            self.toolbar.show()
+        else :
+            self.toolbar.hide()
+
     def toggle_quickadd(self,widget) :
         if widget.get_active() :
             self.quickadd_pane.show()
         else :
             self.quickadd_pane.hide()
-            
+    
+    def canonical_date(self, arg) :
+        """
+        Transform "arg" in a valid yyyy-mm-dd date or return None.
+        "arg" can be a yyyy-mm-dd, yyyymmdd, mmdd, today or a weekday name.
+        """
+        day_names_en = ["monday", "tuesday", "wednesday", "thursday",
+                        "friday", "saturday", "sunday"]
+        day_names = [_("monday"), _("tuesday"), _("wednesday"),
+                     _("thursday"), _("friday"), _("saturday"),
+                     _("sunday")]
+        if re.match(r'\d{4}-\d{2}-\d{2}', arg) :
+            date = arg
+        elif arg.isdigit() :
+            if len(arg) == 8 :
+                date = "%s-%s-%s" % (arg[:4],arg[4:6],arg[6:])
+            elif len(arg) == 4 :
+                year = datetime.date.today().year
+                date = "%i-%s-%s" % (year,arg[:2],arg[2:])
+        elif arg.lower() == "today" or arg.lower() == _("today"):
+            today = datetime.date.today()
+            year = today.year
+            month = today.month
+            day = today.day
+            date = "%i-%i-%i" % (year,month,day)
+        elif arg.lower() == "tomorrow" or\
+          arg.lower() == _("tomorrow") :
+            today = datetime.date.today()
+            tomorrow = today + datetime.timedelta(days=1)
+            year = tomorrow.year
+            month = tomorrow.month
+            day = tomorrow.day
+            date = "%i-%i-%i" % (year,month,day)
+        elif arg.lower() in day_names_en or arg.lower() in day_names:
+            today = datetime.date.today()
+            today_day = today.weekday()
+            if arg.lower() in day_names_en :
+                arg_day = day_names_en.index(arg)
+            else :
+                arg_day = day_names.index(arg)
+            if arg_day > today_day :
+                delta = datetime.timedelta(days = arg_day-today_day)
+            else :
+                delta = datetime.timedelta(days = arg_day-today_day+7)
+            next_date = today + delta
+            year = next_date.year
+            month = next_date.month
+            day = next_date.day
+            date = "%i-%i-%i" % (year,month,day)
+        else :
+            return None
+        if self.is_date_valid(date) :
+            return date
+        else :
+            return None
+    
+    def is_date_valid(self, fulldate) :
+        """
+        Return True if the date exists. False else.
+        "fulldate" is yyyy-mm-dd
+        """
+        splited_date = fulldate.split("-")
+        if len(splited_date) != 3 :
+            return False
+        year,month,day = splited_date
+        try :
+            date = datetime.date(int(year),int(month),int(day))
+        except ValueError :
+            return False
+        else :
+            return True
+    
     def quickadd(self,widget) : #pylint: disable-msg=W0613
         text = self.quickadd_entry.get_text()
+        due_date = None
+        defer_date = None
         if text :
             tags,notagonly = self.get_selected_tags() #pylint: disable-msg=W0612
+            # Get tags in the title
+            for match in re.findall(r'[\s](@[^@,\s]+)',text) :
+                tags.append(GTG.core.tagstore.Tag(match))
+                # Remove the @
+                #text =text.replace(match,match[1:],1)
+            # Get attributes
+            regexp = r'([\s]*)([a-zA-Z0-9_-]+):([^\s]+)'
+            for spaces,attribute, args in re.findall(regexp, text) :
+                valid_attribute = True
+                if attribute.lower() == "tags" or attribute.lower() == _("tags"):
+                    for tag in args.split(",") :
+                        tags.append(GTG.core.tagstore.Tag("@"+tag))
+                elif attribute.lower() == "defer" or attribute.lower() == _("defer"):
+                    defer_date = self.canonical_date(args)
+                    if defer_date is None :
+                        valid_attribute = False
+                elif attribute.lower() == "due" or attribute.lower() == _("due"):
+                    due_date = self.canonical_date(args)
+                    if due_date is None :
+                        valid_attribute = False
+                else :
+                    # attribute is unknown
+                    valid_attribute = False
+                if valid_attribute :
+                    # if the command is valid we have to remove it
+                    # from the task title
+                    text = text.replace("%s%s:%s" % (spaces,attribute,args), "")
+            # Create the new task
             task = self.req.new_task(tags=tags,newtask=True)
-            task.set_title(text)
+            if text != "" :
+                task.set_title(text)
+            if not due_date is None :
+                task.set_due_date(due_date)
+            if not defer_date is None :
+                task.set_start_date(defer_date)
+            id_toselect = task.get_id()
+            #############
             self.quickadd_entry.set_text('')
-            self.do_refresh()
+            # Refresh the treeview
+            self.do_refresh(toselect=id_toselect)
+            #self.select_task(id_toselect)
+            
+    #This works only in the main task_tview
+    #If it cannot find the requested task, nothing is selected
+    def select_task(self,id_toselect) :
+        #We will loop over all task_tview element to find the newly added one
+        model = self.task_tview.get_model()
+        tempit = model.get_iter_first()
+        it = None
+        while (tempit and not it) :
+            if tempit :
+                tid = model.get_value(tempit, 0)
+                if tid == id_toselect :
+                    it = tempit
+            #First we try to see if there is child task
+            tempit2 = model.iter_children(tempit)
+            #if no children, then take the tasks on the same level
+            if not tempit2 :
+                tempit2 = model.iter_next(tempit)
+            #if no task on the same level, go back to the parent
+            #and then to the next task on the parent level
+            if not tempit2 :
+                tempit2 = model.iter_parent(tempit)
+                if tempit2 :
+                    tempit2 = model.iter_next(tempit2)
+            tempit = tempit2
+        if it :
+            selection = self.task_tview.get_selection() 
+            selection.select_iter(it)
+
     
     
-    def do_refresh(self,sender=None,param=None) : #pylint: disable-msg=W0613
+    def do_refresh(self,sender=None,param=None,toselect=None) : #pylint: disable-msg=W0613
         #We ask to do the refresh in a gtk thread
         #We use a lock_lock like described in 
         #http://ploum.frimouvy.org/?202-the-signals-and-threads-flying-circus
         if self.refresh_lock_lock.acquire(False) :
-            gobject.idle_add(self.refresh_tb,param)
+            gobject.idle_add(self.refresh_tb,sender,toselect)
+        #If we have something toselect, we cannot skip the refresh
+        elif toselect :
+            gobject.idle_add(self.select_task,toselect)
 
     #If a task asked for the refresh, we don't refresh it to avoid a loop
     #New use refresh_tb directly, use "do_refresh"
-    def refresh_tb(self,fromtask=None):
+    def refresh_tb(self,fromtask=None,toselect=None):
         self.refresh_lock.acquire()
-        self.refresh_lock_lock.release()
-        current_pane = self.main_pane.get_child()
-        if self.noteview :
-            if current_pane == self.task_tview :
-                self.main_pane.remove(current_pane)
-                self.main_pane.add(self.note_tview)
-            self.refresh_note()
-        else :
-            if current_pane == self.note_tview :
-                self.main_pane.remove(current_pane)
-                self.main_pane.add(self.task_tview)
-            self.refresh_list()
-        self.refresh_closed()
-        self.refresh_tags()
-        #Refreshing the opened editors
-        for uid in self.opened_task :
-            if uid != fromtask :
-                self.opened_task[uid].refresh_editor()
-        self.refresh_lock.release()
+        try :
+            self.refresh_lock_lock.release()
+            current_pane = self.main_pane.get_child()
+            if self.noteview :
+                if current_pane == self.task_tview :
+                    self.main_pane.remove(current_pane)
+                    self.main_pane.add(self.note_tview)
+                self.refresh_note()
+            else :
+                if current_pane == self.note_tview :
+                    self.main_pane.remove(current_pane)
+                    self.main_pane.add(self.task_tview)
+                self.refresh_list(toselect=toselect)
+            self.refresh_closed()
+            self.refresh_tags()
+            #Refreshing the opened editors
+            for uid in self.opened_task :
+                if uid != fromtask :
+                    self.opened_task[uid].refresh_editor()
+        finally :
+            self.refresh_lock.release()
 
 
     #We refresh the tag list. Not needed very often
@@ -539,11 +769,17 @@ class TaskBrowser:
         self.tag_ts.clear()
         alltag       = self.req.get_alltag_tag()
         notag        = self.req.get_notag_tag()
-
-        count_all_task = len(self.req.get_tasks_list())
-        count_no_tags  = len(self.req.get_tasks_list(notag_only=True))
-        self.tag_ts.append([alltag,None,"<span weight=\"bold\">All tags</span>",str(count_all_task),False])
-        self.tag_ts.append([notag,None,"<span weight=\"bold\">Tasks without tags</span>",str(count_no_tags),False])
+        if self.workview :
+            count_all_task = len(self.req.get_active_tasks_list(workable=True))
+            count_no_tags  = len(self.req.get_active_tasks_list(notag_only=True,\
+                                                                workable=True))
+        else :
+            count_all_task = len(self.req.get_tasks_list(started_only=False))
+            count_no_tags  = len(self.req.get_tasks_list(notag_only=True,\
+                                                         started_only=False))
+            
+        self.tag_ts.append([alltag,None,_("<span weight=\"bold\">All tags</span>"),str(count_all_task),False])
+        self.tag_ts.append([notag,None,_("<span weight=\"bold\">Tasks without tags</span>"),str(count_no_tags),False])
         self.tag_ts.append([None,None,"","",True])
 
         tags = self.req.get_used_tags()
@@ -552,7 +788,10 @@ class TaskBrowser:
 
         for tag in tags:
             color = tag.get_attribute("color")
-            count = len(self.req.get_tasks_list(tags=[tag]))
+            if self.workview :
+                count = len(self.req.get_active_tasks_list(tags=[tag],workable=True))
+            else :
+                count = len(self.req.get_tasks_list(started_only=False,tags=[tag]))
             #We display the tags without the "@" (but we could)
             if count != 0:
                 self.tag_ts.append([tag,color,tag.get_name()[1:], str(count), False])
@@ -592,7 +831,7 @@ class TaskBrowser:
         
     #refresh list build/refresh your TreeStore of task
     #to keep it in sync with your self.projects   
-    def refresh_list(self,a=None) : #pylint: disable-msg=W0613
+    def refresh_list(self,a=None,toselect=None) : #pylint: disable-msg=W0613
         
         # Save collapsed rows
         self.task_ts.foreach(self.update_collapsed_row, None)
@@ -635,12 +874,12 @@ class TaskBrowser:
             
         #Set the title of the window :
         if nbr_of_tasks == 0 :
-            parenthesis = "(no active task)"
+            parenthesis = _("(no active tasks)")
         elif nbr_of_tasks == 1 :
-            parenthesis = "(1 active task)"
+            parenthesis = _("(1 active task)")
         else :
-            parenthesis = "(%s actives tasks)"%nbr_of_tasks
-        self.window.set_title("Getting Things Gnome! %s"%parenthesis)
+            parenthesis = "(%s active tasks)"%nbr_of_tasks
+        self.window.set_title("Getting Things GNOME! %s"%parenthesis)
         self.task_tview.set_model(new_taskts)
         self.task_ts = new_taskts
         #We expand all the we close the tasks who were not saved as "expanded"
@@ -655,8 +894,10 @@ class TaskBrowser:
                     self.sort_tasklist_rows(self.priv["tasklist"]["sort_column"], \
                                             self.priv["tasklist"]["sort_order"])
         #We reselect the selected tasks
-        selection = self.task_tview.get_selection()
-        if t_path :
+        if toselect :
+            self.select_task(toselect)
+        elif t_path :
+            selection = self.task_tview.get_selection()
             for i in t_path :
                 selection.select_path(i)
                 
@@ -690,7 +931,7 @@ class TaskBrowser:
                                                     notag_only=notag_only)
         for tid in closed_tasks :
             t              = self.req.get_task(tid)
-            title_str      = t.get_title()
+            title_str      = saxutils.escape(t.get_title())
             closeddate     = t.get_closed_date()
             closeddate_str = closeddate
             tags           = t.get_tags()
@@ -722,7 +963,7 @@ class TaskBrowser:
         notes = self.req.get_notes_list(tags=tag_list, notag_only=notag_only)
         for tid in notes :
             t              = self.req.get_task(tid)
-            title_str      = t.get_title()
+            title_str      = saxutils.escape(t.get_title())
             self.note_ts.append(None,[tid,t.get_color(),title_str])
         note_selection = self.note_tview.get_selection()
         if d_path :
@@ -741,9 +982,10 @@ class TaskBrowser:
             title_str = self.__build_task_title(task, st_count, extended=False)
         else :
             title_str = self.__build_task_title(task, st_count, extended=False)
+    
 
         # Extract data
-        title       = task.get_title() 
+        title       = saxutils.escape(task.get_title()) 
         duedate_str = task.get_due_date()
         left_str    = task.get_days_left()
         tags        = task.get_tags()
@@ -782,13 +1024,25 @@ class TaskBrowser:
             self.task_tview.get_selection().unselect_all()
             self.note_tview.get_selection().unselect_all()
             if task.get_status() == "Dismiss" :
+                self.wTree.get_widget("ctcm_mark_as_not_done").set_sensitive(False)
+                self.wTree.get_widget("ctcm_undismiss").set_sensitive(True)
                 self.dismissbutton.set_label(GnomeConfig.MARK_UNDISMISS)
                 self.donebutton.set_label(GnomeConfig.MARK_DONE)
+                self.donebutton.set_tooltip_text(GnomeConfig.MARK_DONE_TOOLTIP)
                 self.dismissbutton.set_icon_name("gtg-task-undismiss")
+                self.dismissbutton.set_tooltip_text(GnomeConfig.MARK_UNDISMISS_TOOLTIP)
+#                self.editbutton.connect('clicked', self.on_edit_done_task)
+#                self.edit_mi.connect('activate', self.on_edit_done_task)
             else :
+                self.wTree.get_widget("ctcm_mark_as_not_done").set_sensitive(True)
+                self.wTree.get_widget("ctcm_undismiss").set_sensitive(False)
                 self.donebutton.set_label(GnomeConfig.MARK_UNDONE)
+                self.donebutton.set_tooltip_text(GnomeConfig.MARK_UNDONE_TOOLTIP)
                 self.dismissbutton.set_label(GnomeConfig.MARK_DISMISS)
+                self.dismissbutton.set_tooltip_text(GnomeConfig.MARK_DISMISS_TOOLTIP)
                 self.donebutton.set_icon_name("gtg-task-undone")
+#                self.editbutton.connect('clicked', self.on_edit_active_task)
+#                self.edit_mi.connect('activate', self.on_edit_active_task)
                 
     #This function is called when the selection change in the active task view
     #It will displays the selected task differently
@@ -801,6 +1055,7 @@ class TaskBrowser:
             self.taskdone_tview.get_selection().unselect_all()
             self.note_tview.get_selection().unselect_all()
             self.donebutton.set_label(GnomeConfig.MARK_DONE)
+            self.donebutton.set_tooltip_text(GnomeConfig.MARK_DONE_TOOLTIP)
             self.dismissbutton.set_label(GnomeConfig.MARK_DISMISS)
         #We reset the previously selected task
         if self.selected_rows and self.task_ts.iter_is_valid(self.selected_rows):
@@ -838,20 +1093,21 @@ class TaskBrowser:
         return count
     
     def __build_task_title(self,task,count,extended=False):
+        simple_title = saxutils.escape(task.get_title())
         if extended :
             excerpt = task.get_excerpt(lines=2)
             if excerpt.strip() != "" :
-                title   = "<b><big>%s</big></b>\n<small>%s</small>" %(task.get_title(),excerpt)
+                title   = "<b><big>%s</big></b>\n<small>%s</small>" %(simple_title,excerpt)
             else : 
-                title   = "<b><big>%s</big></b>" %task.get_title()
+                title   = "<b><big>%s</big></b>" %simple_title 
         else :
             if (not self.workview):
                 if count == 0:
-                    title = "<span>%s</span>" % (task.get_title() )
+                    title = "<span>%s</span>" % (simple_title )
                 else:
-                    title = "<span>%s (%s)</span>" % (task.get_title(), count )
+                    title = "<span>%s (%s)</span>" % (simple_title , count )
             else:
-                title = task.get_title()
+                title = simple_title 
         return title
 
     #If a Task editor is already opened for a given task, we present it
@@ -926,6 +1182,10 @@ class TaskBrowser:
                 self.taskpopup.popup( None, None, None, event.button, time)
             return 1
 
+    def on_task_treeview_key_press_event(self,treeview,event) :
+        if gtk.gdk.keyval_name(event.keyval) == "Delete":
+            self.on_delete_task()
+
     def on_closed_task_treeview_button_press_event(self,treeview,event) :
         if event.button == 3:
             x = int(event.x)
@@ -939,6 +1199,10 @@ class TaskBrowser:
                 self.closedtaskpopup.popup( None, None, None, event.button, time)
             return 1
             
+    def on_closed_task_treeview_key_press_event(self,treeview,event) :
+        if gtk.gdk.keyval_name(event.keyval) == "Delete":
+            self.on_delete_task()
+
     def on_add_task(self,widget,status=None) : #pylint: disable-msg=W0613
         tags,notagonly = self.get_selected_tags() #pylint: disable-msg=W0612
         task = self.req.new_task(tags=tags,newtask=True)
@@ -1086,7 +1350,7 @@ class TaskBrowser:
         
         
     def on_edit_active_task(self,widget,row=None ,col=None) : #pylint: disable-msg=W0613
-        tid = self.get_selected_task(self.task_tview)
+        tid = self.get_selected_task()
         if tid :
             self.open_task(tid)
     def on_edit_done_task(self,widget,row=None ,col=None) : #pylint: disable-msg=W0613
@@ -1105,7 +1369,7 @@ class TaskBrowser:
         self.tid_todelete = None
         self.do_refresh()
         
-    def on_delete_task(self,widget,tid=None) : #pylint: disable-msg=W0613
+    def on_delete_task(self,widget=None,tid=None) : #pylint: disable-msg=W0613
         #If we don't have a parameter, then take the selection in the treeview
         if not tid :
             #tid_to_delete is a [project,task] tuple
@@ -1176,7 +1440,8 @@ class TaskBrowser:
         tag_col.set_expand            (True)
         self.tag_tview.append_column  (tag_col)
         # Global treeview properties
-        self.tag_tview.set_row_separator_func(self.tag_separator_filter)
+        self.tag_tview.set_row_separator_func (self.tag_separator_filter)
+        self.tag_tview.set_headers_visible    (False)
 
     def cmp_duedate_str(self, key1, key2):
         if self.priv["tasklist"]["sort_order"] == gtk.SORT_ASCENDING:
@@ -1217,7 +1482,7 @@ class TaskBrowser:
 
         # Determine row sorting depending on column
         if column == self.priv["tasklist"]["columns"][self.TASKLIST_COL_TITLE]:
-            cmp_func = lambda x,y: cmp(x.lower(),y.lower())
+            cmp_func = lambda x,y: locale.strcoll(x.lower(),y.lower())
             sort_key = lambda x:x[self.TASK_MODEL_TITLE]
         else:
             cmp_func = self.cmp_duedate_str
@@ -1246,7 +1511,7 @@ class TaskBrowser:
         self.TASKLIST_COL_TAGS = 0
         tag_col     = gtk.TreeViewColumn()
         render_tags = CellRendererTags()
-        tag_col.set_title             ("Tags")
+        tag_col.set_title             (_("Tags"))
         tag_col.pack_start            (render_tags, expand=False)
         tag_col.add_attribute         (render_tags, "tag_list", self.TASK_MODEL_TAGS)
         render_tags.set_property      ('xalign', 0.0)
@@ -1261,7 +1526,7 @@ class TaskBrowser:
         self.TASKLIST_COL_TITLE = 1
         title_col   = gtk.TreeViewColumn()
         render_text = gtk.CellRendererText()
-        title_col.set_title           ("Title")
+        title_col.set_title           (_("Title"))
         title_col.pack_start          (render_text, expand=False)
         title_col.add_attribute       (render_text, "markup", self.TASK_MODEL_TITLE_STR)
         title_col.set_resizable       (True)
@@ -1280,7 +1545,7 @@ class TaskBrowser:
         self.TASKLIST_COL_DDATE = 2
         ddate_col   = gtk.TreeViewColumn()
         render_text = gtk.CellRendererText()
-        ddate_col.set_title           ("Due date")
+        ddate_col.set_title           (_("Due date"))
         ddate_col.pack_start          (render_text, expand=False)
         ddate_col.add_attribute       (render_text, "markup", self.TASK_MODEL_DDATE_STR)
         ddate_col.set_resizable       (False)
@@ -1294,7 +1559,7 @@ class TaskBrowser:
         self.TASKLIST_COL_DLEFT = 3
         dleft_col   = gtk.TreeViewColumn()
         render_text = gtk.CellRendererText()
-        dleft_col.set_title           ("Days left")
+        dleft_col.set_title           (_("Days left"))
         dleft_col.pack_start          (render_text, expand=False)
         dleft_col.add_attribute       (render_text, "markup", self.TASK_MODEL_DLEFT_STR)
         dleft_col.set_resizable       (False)
@@ -1317,7 +1582,7 @@ class TaskBrowser:
         self.CTASKLIST_COL_TAGS = 0
         tag_col     = gtk.TreeViewColumn()
         render_tags = CellRendererTags()
-        tag_col.set_title             ("Tags")
+        tag_col.set_title             (_("Tags"))
         tag_col.pack_start            (render_tags, expand=False)
         tag_col.add_attribute         (render_tags, "tag_list", self.CTASKS_MODEL_TAGS)
         render_tags.set_property      ('xalign', 0.0)
@@ -1332,7 +1597,7 @@ class TaskBrowser:
         self.CTASKLIST_COL_DDATE = 1
         ddate_col    = gtk.TreeViewColumn()
         render_text  = gtk.CellRendererText()
-        ddate_col.set_title                ("Closing date")
+        ddate_col.set_title                (_("Closing date"))
         ddate_col.pack_start               (render_text  , expand=True)
         ddate_col.set_attributes           (render_text  , markup=self.CTASKS_MODEL_DDATE_STR)
         ddate_col.set_sort_column_id       (self.CTASKS_MODEL_DDATE)
@@ -1344,7 +1609,7 @@ class TaskBrowser:
         self.CTASKLIST_COL_TITLE = 2
         title_col    = gtk.TreeViewColumn()
         render_text  = gtk.CellRendererText()
-        title_col.set_title                ("Title")
+        title_col.set_title                (_("Title"))
         title_col.pack_start               (render_text  , expand=True)
         title_col.set_attributes           (render_text  , markup=self.CTASKS_MODEL_TITLE)
         title_col.set_sort_column_id       (self.CTASKS_MODEL_TITLE)
@@ -1360,7 +1625,7 @@ class TaskBrowser:
         # Title column
         title_col    = gtk.TreeViewColumn()
         render_text  = gtk.CellRendererText()
-        title_col.set_title                ("Notes")
+        title_col.set_title                (_("Notes"))
         title_col.pack_start               (render_text  , expand=True)
         title_col.set_attributes           (render_text  , markup=self.CTASKS_MODEL_TITLE)
         title_col.set_sort_column_id       (self.CTASKS_MODEL_TITLE)
@@ -1374,4 +1639,5 @@ class TaskBrowser:
     ######Closing the window
     def close(self,widget=None) : #pylint: disable-msg=W0613
         #Saving is now done in main.py
+        self.on_delete(None, None)
         gtk.main_quit()
