@@ -24,12 +24,20 @@ class GnomeConfig :
     current_rep = os.path.dirname(os.path.abspath(__file__))
     GLADE_FILE    = os.path.join(current_rep,"taskeditor.glade")
     
-    MARK_DONE = "Mark as done"
-    MARK_UNDONE = "Mark as not done"
-    MARK_DISMISS = "Dismiss"
-    MARK_UNDISMISS = "Undismiss"
-    KEEP_NOTE = "Keep as Note"
-    MAKE_TASK = "Make a Task"
+    MARK_DONE = _("Mark as done")
+    MARK_UNDONE = _("Mark as not done")
+    MARK_DISMISS = _("Dismiss")
+    MARK_UNDISMISS = _("Undismiss")
+    KEEP_NOTE = _("Keep as Note")
+    MAKE_TASK = _("Make a Task")
+    
+    MARK_DONE_TOOLTIP = _("Mark this task as done")
+    MARK_UNDONE_TOOLTIP = _("Mark this task as to be done")
+    MARK_DISMISS_TOOLTIP = _("Mark this task as not to be done anymore")
+    MARK_UNDISMISS_TOOLTIP = _("Mark this task as to be done")
+    DELETE_TOOLTIP = _("Permanently remove this task")
+    SUBTASK_TOOLTIP = _("Insert a subtask in this task")
+    TAG_TOOLTIP = _("Insert a tag in this task")
     
     #Number of second between to save in the task editor
     SAVETIME = 10
