@@ -24,6 +24,7 @@
 #The rest are the logic of the widget : date changing widgets, buttons, ...
 import sys
 import time
+from datetime import date
 
 from GTG.taskeditor          import GnomeConfig
 from GTG.tools               import dates
@@ -52,6 +53,12 @@ class TaskEditor :
         self.cal_tree = gtk.glade.XML(self.gladefile, "calendar")
         self.donebutton = self.wTree.get_widget("mark_as_done_editor")
         self.dismissbutton = self.wTree.get_widget("dismiss_editor")
+        self.deletebutton = self.wTree.get_widget("delete_editor")
+        self.deletebutton.set_tooltip_text(GnomeConfig.DELETE_TOOLTIP)
+        self.deletebutton = self.wTree.get_widget("insert_subtask")
+        self.deletebutton.set_tooltip_text(GnomeConfig.SUBTASK_TOOLTIP)
+        self.deletebutton = self.wTree.get_widget("inserttag")
+        self.deletebutton.set_tooltip_text(GnomeConfig.TAG_TOOLTIP)
         #Create our dictionay and connect it
         dic = {
                 "mark_as_done_clicked"  : self.change_status,
@@ -69,7 +76,7 @@ class TaskEditor :
         self.wTree.signal_autoconnect(dic)
         cal_dic = {
                 "on_nodate"             : self.nodate_pressed,
-                "on_dayselected"        : self.day_selected,
+                #"on_dayselected"        : self.day_selected,
                 "on_dayselected_double" : self.day_selected_double,
         }
         self.cal_tree.signal_autoconnect(cal_dic)
@@ -93,6 +100,9 @@ class TaskEditor :
         scrolled.add(self.textview)
         #Voila! it's done
         self.calendar       = self.cal_tree.get_widget("calendar")
+        self.cal_widget       = self.cal_tree.get_widget("calendar1")
+        #self.cal_widget.set_property("no-month-change",True)
+        self.sigid = None
         self.duedate_widget = self.wTree.get_widget("duedate_entry")
         self.startdate_widget = self.wTree.get_widget("startdate_entry")
         self.dayleft_label  = self.wTree.get_widget("dayleft")
@@ -141,6 +151,9 @@ class TaskEditor :
             subtasks = task.get_subtasks_tid()
             if subtasks :
                 self.textview.insert_subtasks(subtasks)
+        #We select the title if it's a new task
+        if self.task.is_new() :
+            self.textview.select_title()
         self.textview.modified(full=True)
         self.window.connect("destroy", self.destruction)
         
@@ -148,6 +161,7 @@ class TaskEditor :
         #Putting the refresh callback at the end make the start a lot faster
         self.textview.refresh_callback(self.refresh_editor)
         self.refresh_editor()
+        self.textview.grab_focus()
 
         self.window.show()
 
@@ -176,15 +190,21 @@ class TaskEditor :
         status = self.task.get_status() 
         if status == "Dismiss" :
             self.donebutton.set_label(GnomeConfig.MARK_DONE)
+            self.donebutton.set_tooltip_text(GnomeConfig.MARK_DONE_TOOLTIP)
             self.dismissbutton.set_label(GnomeConfig.MARK_UNDISMISS)
+            self.dismissbutton.set_tooltip_text(GnomeConfig.MARK_UNDISMISS_TOOLTIP)
             self.dismissbutton.set_icon_name("gtg-task-undismiss")
         elif status == "Done" :
             self.donebutton.set_label(GnomeConfig.MARK_UNDONE)
+            self.donebutton.set_tooltip_text(GnomeConfig.MARK_UNDONE_TOOLTIP)
             self.donebutton.set_icon_name("gtg-task-undone")
             self.dismissbutton.set_label(GnomeConfig.MARK_DISMISS)
+            self.dismissbutton.set_tooltip_text(GnomeConfig.MARK_DISMISS_TOOLTIP)
         else :
             self.donebutton.set_label(GnomeConfig.MARK_DONE)
+            self.donebutton.set_tooltip_text(GnomeConfig.MARK_DONE_TOOLTIP)
             self.dismissbutton.set_label(GnomeConfig.MARK_DISMISS)
+            self.dismissbutton.set_tooltip_text(GnomeConfig.MARK_UNDISMISS_TOOLTIP)
             
         if status == "Note" :
             self.donebutton.hide()
@@ -204,15 +224,15 @@ class TaskEditor :
                 #refreshing the day left label
                 result = self.task.get_days_left()
                 if result == 1 :
-                    txt = "Due tomorrow !"
+                    txt = _("Due tomorrow !")
                 elif result > 0 :
-                    txt = "%s days left" %result
+                    txt = _("%s days left") %result
                 elif result == 0 :
-                    txt = "Due today !"
+                    txt = _("Due today !")
                 elif result == -1 :
-                    txt = "Due for yesterday"
+                    txt = _("Due for yesterday")
                 elif result < 0 :
-                    txt = "Was %s days ago" %result
+                    txt = _("Was %s days ago") %result
                 self.dayleft_label.set_markup("<span color='#666666'>"+txt+"</span>")    
         elif self.duedate_widget.get_text() != ''  :
             self.dayleft_label.set_text('')
@@ -287,7 +307,23 @@ class TaskEditor :
         gdk.pointer_grab(self.calendar.window, True,gdk.BUTTON1_MASK|gdk.MOD2_MASK)
         #we will close the calendar if the user clic outside
         self.__opened_date = data
+        if self.__opened_date == "due" :
+            toset = self.task.get_due_date()
+        elif self.__opened_date == "start" :
+            toset = self.task.get_start_date()
+        if toset :
+            y,m,d = toset.split("-")
+        else :
+            dd = date.today()
+            y = dd.year
+            m = dd.month
+            d = dd.day
+        #Else, we set the widget to today's date
+        
+        self.cal_widget.select_month(int(m)-1,int(y))
+        self.cal_widget.select_day(int(d))
         self.calendar.connect('button-press-event', self.__focus_out)
+        self.sigid = self.cal_widget.connect("day-selected",self.day_selected)
         
     def day_selected(self,widget) :
         y,m,d = widget.get_date()
@@ -358,19 +394,21 @@ class TaskEditor :
         tid = subt.get_id()
         return tid
         
-    def insert_subtask(self,widget) :
-        itera =  self.textview.get_insert()
+    def insert_subtask(self,widget) : #pylint: disable-msg=W0613
         self.textview.insert_newtask()
+        self.textview.grab_focus()
         
-    def inserttag_clicked(self,widget) :
+    def inserttag_clicked(self,widget) : #pylint: disable-msg=W0613
         itera = self.textview.get_insert()
         if itera.starts_line() :
             self.textview.insert_text("@",itera)
         else :
             self.textview.insert_text(" @",itera)
+        self.textview.grab_focus()
         
-    def inserttag(self,widget,tag) :
+    def inserttag(self,widget,tag) : #pylint: disable-msg=W0613
         self.textview.insert_tags([tag])
+        self.textview.grab_focus()
     
     def save(self) :
         self.task.set_title(self.textview.get_title())
@@ -423,6 +461,9 @@ class TaskEditor :
         self.__opened_date = ''
         gtk.gdk.pointer_ungrab()
         self.calendar.grab_remove()
+        if self.sigid :
+            self.cal_widget.disconnect(self.sigid)
+            self.sigid = None
         
 
     
