@@ -33,8 +33,10 @@ import gobject
 import pango
 
 from GTG.taskeditor import taskviewserial
+from GTG.tools import openurl
 
-separators = [' ','.',',','/','\n','\t','!','?',';']
+separators = [' ','.',',','/','\n','\t','!','?',';','\0']
+url_separators = [' ',',','\n','\t','\0']
 
 bullet1 = '→'
 bullet2 = '↳'
@@ -247,11 +249,15 @@ class TaskView(gtk.TextView):
         #We cannot have two tags with the same name
         #That's why the link tag has no name
         #but it has a "is_anchor" property
-        task = self.req.get_task(anchor)
-        if task and task.get_status() == "Active" :
+        if typ == "http" :
             linktype = 'link'
+        #By default, the type is a subtask
         else :
-            linktype = 'done'
+            task = self.req.get_task(anchor)
+            if task and task.get_status() == "Active" :
+                linktype = 'link'
+            else :
+                linktype = 'done'
         tag = b.create_tag(None, **self.get_property(linktype)) #pylint: disable-msg=W0142
         tag.set_data('is_anchor', True)
         tag.set_data('link',anchor)
@@ -263,11 +269,24 @@ class TaskView(gtk.TextView):
         
     #Apply the tag tag to a set of TextMarks (not Iter)
     def apply_tag_tag(self,buff,tag,s,e) :
-        texttag = buff.create_tag(None,**self.get_property('tag'))#pylint: disable-msg=W0142
-        texttag.set_data('is_tag', True)
-        texttag.set_data('tagname',tag)
-        #This one is for marks
-        self.__apply_tag_to_mark(s,e,tag=texttag)
+        ss = buff.get_iter_at_mark(s)
+        ee = buff.get_iter_at_mark(e)
+        #If the tag is already applied, we do nothing
+        t_list = ss.get_tags()
+        texttag = None
+        already = False
+        for t in t_list :
+            if t.get_data('is_tag') and t.get_data('tagname') == tag :
+                texttag = t
+                if ss.begins_tag(t) and ee.ends_tag(t) :
+                    already = True
+        if not texttag :
+            texttag = buff.create_tag(None,**self.get_property('tag'))#pylint: disable-msg=W0142
+            texttag.set_data('is_tag', True)
+            texttag.set_data('tagname',tag)
+            #This one is for marks
+        if not already :
+            self.__apply_tag_to_mark(s,e,tag=texttag)
         
     #Apply the tag tag to a set of TextMarks (not Iter)
     #Also change the subtask title if needed
@@ -297,7 +316,8 @@ class TaskView(gtk.TextView):
     def insert_subtasks(self,st_list) :
         for tid in st_list :
             line_nbr = self.buff.get_end_iter().get_line()
-            self.write_subtask(self.buff,line_nbr,tid)
+            #Warning, we have to take the next line !
+            self.write_subtask(self.buff,line_nbr+1,tid)
             
     #Insert a list of tag in the first line of the buffer
     def insert_tags(self,tag_list) :
@@ -312,6 +332,16 @@ class TaskView(gtk.TextView):
                 #Now we should check if the current char is a separator or not
                 #Currently, we insert a space
                 self.insert_text(" ",firstline)
+        #Now we check if this newline is empty (it contains only " " and ",")
+#        if newline :
+#            endline = firstline.copy()
+#            if not endline.ends_line() :
+#                endline.forward_to_line_end()
+#            text = self.buff.get_text(firstline,endline)
+#            if not text.strip(", ") :
+#                newline = False
+#                firstline.forward_to_line_end()
+        #Now we can process
         if newline :
             firstline = self.buff.get_iter_at_line(0)
             firstline.forward_to_line_end()
@@ -320,12 +350,18 @@ class TaskView(gtk.TextView):
         line_mark = self.buff.create_mark("firstline",firstline,False)
         #self.tv.insert_at_mark(buf,line_mark,"\n")
         for t in tag_list :
-            it = self.buff.get_iter_at_mark(line_mark)
             self.insert_at_mark(self.buff,line_mark,t)
             self.insert_at_mark(self.buff,line_mark,",")
         self.buff.delete_mark(line_mark)
         self.modified(full=True)
         
+    #this function select and highligth the title (first line)
+    def select_title(self) :
+        start = self.buff.get_start_iter()
+        stop = start.copy()
+        stop.forward_to_line_end()
+        self.buff.select_range(start, stop)
+        
  ##### The "Get text" group #########
     #Get the complete serialized text
     #But without the title
@@ -385,11 +421,18 @@ class TaskView(gtk.TextView):
         else :
             #We analyse only the current line
             local_start = cursor_iter.copy()
-            local_start.backward_line()
+            local_start.set_line(local_start.get_line())
             local_end = cursor_iter.copy()
             local_end.forward_lines(2)
         #if full=False we detect tag only on the current line
+        
+        #The following 3 lines are a quick ugly fix for bug #359469
+#        temp = buff.get_iter_at_line(1)
+#        temp.backward_char()
+#        self._detect_tag(buff,temp,buff.get_end_iter())
+        #This should be the good line
         self._detect_tag(buff,local_start,local_end)
+        self._detect_url(buff,local_start,local_end)
         
         #subt_list = self.get_subtasks()
         #First, we remove the olds tags
@@ -431,6 +474,58 @@ class TaskView(gtk.TextView):
         #Else we save the task anyway (but without refreshing all)
         elif self.save_task :
             self.save_task()
+            
+    #Detect URL in the tasks
+    #It's ugly...
+    def _detect_url(self,buff,start,end) :
+        #firtst, we remove the old tags :
+        #subt_list = self.get_subtasks()
+        #First, we remove the olds tags
+        tag_list = []
+        table = buff.get_tag_table()
+        def subfunc(texttag,data=None) : #pylint: disable-msg=W0613
+            if texttag.get_data('is_anchor') :
+                tag_list.append(texttag)
+        table.foreach(subfunc)
+        for t in tag_list :
+            buff.remove_tag(t,start,end)
+        #Now we add the tag URL
+        it = start.copy()
+        prev = start.copy()
+        while (it.get_offset() < end.get_offset()) and (it.get_char() != '\0') :
+            it.forward_word_end()
+            prev = it.copy()
+            prev.backward_word_start()
+            text = buff.get_text(prev,it)
+            if text in ["http","https"] :
+                while it.get_char() not in url_separators and (it.get_char() != '\0') :
+                    it.forward_char()
+                url = buff.get_text(prev,it)
+                if url.startswith("http://") or url.startswith("https://") :
+                    texttag = self.create_anchor_tag(buff,url,text=None,typ="http")
+                    buff.apply_tag(texttag, prev , it)
+            elif text in ["bug","lp","bgo","fdo"] :
+                if it.get_char() == " " :
+                    it.forward_char()
+                if it.get_char() == "#" :
+                    it.forward_char()
+                    while it.get_char().isdigit() and (it.get_char() != '\0') :
+                        it.forward_char()
+                    url = buff.get_text(prev,it)
+                    nbr = url.split("#")[1]
+                    topoint = None
+                    if url.startswith("bug #") or url.startswith("lp #") :
+                        topoint = "https://launchpad.net/bugs/%s" %nbr
+                    elif url.startswith("bgo #") :
+                        topoint = "http://bugzilla.gnome.org/show_bug.cgi?id=%s" %nbr
+                    elif url.startswith("fdo #") :
+                        topoint = "http://bugs.freedesktop.org/show_bug.cgi?id=%s" %nbr
+                    if topoint :
+                        texttag = self.create_anchor_tag(buff,\
+                                                topoint,text=None,typ="http")
+                        buff.apply_tag(texttag, prev , it)
+                
+        
 
     #Detect tags in buff in the regio between start iter and end iter
     def _detect_tag(self,buff,start,end) :
@@ -444,12 +539,13 @@ class TaskView(gtk.TextView):
         #found the beginning of a tag on the nextline
         while (it.get_offset() < end.get_offset()) and (it.get_char() != '\0'):
             if it.begins_tag() :
-                tags = it.get_tags()
+                tags = it.get_toggled_tags(True)
                 for ta in tags :
                     #removing deleted tags
                     if ta.get_data('is_tag') :
                         tagname = ta.get_data('tagname')
                         old_tags.append(tagname)
+                        buff.remove_tag(ta,start,end)
                         table.remove(ta)
                         #Removing the marks if they exist
                         mark1 = buff.get_mark(tagname)
@@ -457,7 +553,7 @@ class TaskView(gtk.TextView):
                             offset1 = buff.get_iter_at_mark(mark1).get_offset()
                             if start.get_offset() <= offset1 <= end.get_offset() :
                                 buff.delete_mark_by_name(tagname)
-                        mark2 = buff.get_mark("%s"%tagname)
+                        mark2 = buff.get_mark("/%s"%tagname)
                         if mark2 :
                             offset2 = buff.get_iter_at_mark(mark2).get_offset()
                             if start.get_offset() <= offset2 <= end.get_offset():
@@ -489,7 +585,6 @@ class TaskView(gtk.TextView):
             if do_word_check:
                 if (word_end.compare(word_start) > 0):
                     my_word = buff.get_text(word_start, word_end)
-                
                     # We do something about it
                     #We want a tag bigger than the simple "@"
                     if len(my_word) > 1 and my_word[0] == '@':
@@ -536,11 +631,17 @@ class TaskView(gtk.TextView):
     #from this selection
     def _delete_range(self,buff,start,end) :
         it = start.copy()
+#        #If we are at the beginning of a mark, put this mark at the end
+#        marks = start.get_marks()
+#        for m in marks :
+#            print m.get_name()
+#            buff.move_mark(m,end)
         #If the begining of the selection is in the middle of an indent
         #We want to start at the begining
         tags = it.get_tags()+it.get_toggled_tags(False)
         for ta in tags :
-            if (ta.get_data('is_indent') and not it.begins_tag(ta)) :
+            if (ta.get_data('is_indent') and not it.begins_tag(ta) \
+                                            and not it.ends_tag(ta)) :
                 it.backward_to_tag_toggle(ta)
                 start.backward_to_tag_toggle(ta)
                 endindent = it.copy()
@@ -552,9 +653,10 @@ class TaskView(gtk.TextView):
                 tags = it.get_tags()
                 for ta in tags :
                     #removing deleted subtasks
+                    #it looks like it works without that.
                     if ta.get_data('is_subtask') :
-                        target = ta.get_data('child')
-                        self.remove_subtask(target)
+#                        target = ta.get_data('child')
+#                        #self.remove_subtask(target)
                         self.refresh_browser()
                     #removing deleted tags
                     if ta.get_data('is_tag') :
@@ -680,6 +782,9 @@ class TaskView(gtk.TextView):
             fitera = self.get_insert()
         #First, find a line without subtask
         line = fitera.get_line()
+        #Avoid the title at all cost
+        if line <= 0 :
+            line = 1
         startl = self.buff.get_iter_at_line(line)
         itera = None
         while not itera :
@@ -742,6 +847,10 @@ class TaskView(gtk.TextView):
         #of the newly inserted \n, it will not move anymore.
         
         itera = buff.get_iter_at_mark(end)
+        #We should never have an indentation at 0.
+        #This is normally not needed and purely defensive
+        if itera.get_line() <= 0 :
+            itera = buff.get_iter_at_line(1)
         start   = buff.create_mark("start",itera,True)
         indentation = ""
         #adding two spaces by level
@@ -804,6 +913,23 @@ class TaskView(gtk.TextView):
         #New line : the user pressed enter !
         #If the line begins with "-", it's a new subtask !
         if tex == '\n' :
+            insert_point = self.buff.create_mark("insert_point",itera,True)
+            #First, we close tag tags.
+            #If we are at the end of a tag, we look for closed tags
+            closed_tag = None
+            cutting_subtask = False
+            if itera.ends_tag() :
+                list_stag = itera.get_toggled_tags(False)
+            #Or maybe we are in the middle of a tag
+            else :
+                list_stag = itera.get_tags()
+            stag = None
+            for t in list_stag :
+                if t.get_data('is_tag') :
+                    closed_tag = t.get_data('tagname')
+                elif t.get_data('is_subtask') :
+                    cutting_subtask = True
+                    closed_tag = t.get_data('child')
             #We add a bullet list but not on the first line
             #Because it's the title
             if line_nbr > 0 :
@@ -819,6 +945,9 @@ class TaskView(gtk.TextView):
                         #Here, we should increment indent level
                         self.insert_indent(self.buff,end_i,1,enter=True)
                         tv.emit_stop_by_name('insert-text')
+                    else :
+                        self.buff.insert(itera,"\n")
+                        tv.emit_stop_by_name('insert-text')
                         
                 #Then, if indent > 0, we increment it
                 #First step : we preserve it.
@@ -830,6 +959,22 @@ class TaskView(gtk.TextView):
                     elif current_indent == 1 :
                         self.insert_indent(self.buff,itera,current_indent)
                         tv.emit_stop_by_name('insert-text')
+                #Then we close the tag tag
+                if closed_tag :
+                    insert_mark = self.buff.get_mark("insert_point")
+                    insert_iter = self.buff.get_iter_at_mark(insert_mark)
+                    self.buff.move_mark_by_name("/%s"%closed_tag,insert_iter)
+                    self.buff.delete_mark(insert_mark)
+                    if cutting_subtask :
+                        cursor = self.buff.get_iter_at_mark(self.buff.get_insert())
+                        endl = cursor.copy()
+                        if not endl.ends_line() :
+                            endl.forward_to_line_end()
+                        text = self.buff.get_text(cursor,endl)
+                        anchor = self.new_subtask_callback(text)
+                        self.buff.create_mark(anchor,cursor,True)
+                        self.buff.create_mark("/%s"%anchor,endl,False)
+                    self.modified(full=True)
         #The user entered something else than \n
         elif tex :
             #We are on an indented line without subtask ? Create it !
@@ -839,8 +984,30 @@ class TaskView(gtk.TextView):
                 self.buff.create_mark(anchor,itera,True)
                 self.buff.create_mark("/%s"%anchor,itera,False)
         self.insert_sigid = self.buff.connect('insert-text', self._insert_at_cursor)
+        self.keypress_sigid = self.connect('key_press_event', self._keypress)
         self.modified_sigid = self.buff.connect("changed" , self.modified)
         
+    def _keypress(self, widget, event):
+        # Check for Ctrl-Return/Enter
+        if event.state & gtk.gdk.CONTROL_MASK and event.keyval in (gtk.keysyms.Return, gtk.keysyms.KP_Enter):
+            buff = self.buff   
+            cursor_mark = buff.get_insert()
+            cursor_iter = buff.get_iter_at_mark(cursor_mark)
+            table = buff.get_tag_table()
+			
+            local_start = cursor_iter.copy()
+
+            for tag in local_start.get_tags():
+                anchor =  tag.get_data('link')
+                typ =  tag.get_data('type')
+                if(anchor):
+                    if typ == "subtask" :
+                        self.open_task(anchor)
+                    elif typ == "http" :
+                        openurl.openurl(anchor)
+					
+            return True
+
     #Deindent the current line of one level
     #If newlevel is set, force to go to that level
     def deindent(self,itera,newlevel=-1) :
@@ -904,6 +1071,8 @@ class TaskView(gtk.TextView):
             if _type == gtk.gdk.BUTTON_RELEASE:
                 if typ == "subtask" :
                     self.open_task(anchor)
+                elif typ == "http" :
+                    openurl.openurl(anchor)
                 else :
                     print "Unknown link type for %s" %anchor
                 self.emit('anchor-clicked', text, anchor, button)
